@@ -106,6 +106,8 @@ struct device devices[] = {
 #define INIT_NOOP
 #endif
 
+#ifdef isc
+
 #ifdef isc2
 #define predefined_devices
 struct device devices[] = {
@@ -120,7 +122,20 @@ struct device devices[] = {
 };
 
 #define INIT_NOOP
-#endif /* isc2 */
+#else /* isc2 */
+
+#define predefined_devices
+struct device devices[] = {
+	{"/dev/rfd0",		'A', GEN},
+	{"/dev/rfd1",		'B', GEN},
+	{"/dev/rdsk/0p1",	'C', GEN},
+	{"/usr/vpix/defaults/C:",'D', GEN, 1},
+	{"$HOME/vpix/C:", 	'E', GEN, 1}
+};
+#define INIT_NOOP
+#endif
+
+#endif /* isc */
 
 #ifdef i370
 #define predefined_devices
@@ -202,6 +217,13 @@ struct device devices[] = {
 #define INIT_NOOP
 #endif /* irix */
 
+#ifdef osf
+#define predefined_devices
+struct device devices[] = {
+  {"/dev/fd0a", 'A',  HD312 } };
+#define INIT_NOOP
+#endif
+
 #ifdef sunos4
 #include <sys/ioctl.h>
 #include <sun/dkio.h>
@@ -504,6 +526,33 @@ static inline int get_parameters(int fd, struct floppy_struct *floppy)
 #endif /* linux */
 
 
+/*** /jes -- for D.O.S. 486 BL DX2/80 ***/
+#ifdef freebsd
+#define predefined_devices
+struct device devices[] = {
+	{"/dev/rfd0.1440", 'A', HD312},
+	{"/dev/rfd1.1200", 'B', HD514},
+	+    {"/dev/sd0s1",    'C', GENHD},
+};
+#define INIT_NOOP
+#endif /* __FreeBSD__ */
+ 
+/*** /jes -- for ALR 486 DX4/100 ***/
+#ifdef netbsd
+#define predefined_devices
+struct device devices[] = {
+	{"/dev/rfd0a", 'A', HD312},
+	{"/dev/rfd0f", 'A', DD312},
+	{"/dev/rfd0f", 'S', DD312},
+	{"/dev/rfd1a", 'B', HD514},
+	{"/dev/rfd1d", 'B', DD514},
+	{"/dev/rfd1d", 'T', DD514},
+	{"/dev/rwd0d", 'C', 16, 0, 0, 0, 0, 63L*512L, DEF_ARG0},
+};
+#define INIT_NOOP
+#endif /* NetBSD */
+
+
 #if (!defined(predefined_devices) && defined (m68000) && defined (sysv))
 #include <sys/gdioctl.h>
 
@@ -525,9 +574,9 @@ int init_geom(int fd, struct device *dev, struct device *orig_dev,
 	if((dev->use_2m & 0x7f) || (dev->ssize & 0x7f))
 		return 1;
 	
-	set_int(&gdbuf.params.cyls,dev->ntracks);
-	set_int(&gdbuf.params.heads,dev->nheads);
-	set_int(&gdbuf.params.psectrk,dev->nsect);
+	SET_INT(gdbuf.params.cyls,dev->ntracks);
+	SET_INT(gdbuf.params.heads,dev->nheads);
+	SET_INT(gdbuf.params.psectrk,dev->nsect);
 	dev->ntracks = gdbuf.params.cyls;
 	dev->nheads = gdbuf.params.heads;
 	dev->nsect = gdbuf.params.psectrk;
