@@ -88,7 +88,7 @@ Stream_t *find_device(char drive, int mode, struct device *out_dev,
 		expand(dev->name,name);
 
 #ifdef USE_XDF
-		Stream = XdfOpen(out_dev, name, mode, errmsg);
+		Stream = XdfOpen(out_dev, name, mode, errmsg, 0);
 #else
 		Stream = 0;
 #endif
@@ -137,7 +137,7 @@ Stream_t *fs_init(char drive, int mode)
 	int media,i;
 	int nhs;
 	int disk_size = 0;	/* In case we don't happen to set this below */
-	unsigned long tot_sectors;
+	size_t tot_sectors;
 	char name[EXPAND_BUF];
 	int cylinder_size;
 	struct device dev;
@@ -188,7 +188,7 @@ Stream_t *fs_init(char drive, int mode)
 		if(boot0.dos4 == 0x29) {
 			This->serialized = 1;
 			tot_sectors = DWORD(bigsect);
-			set_ulong(&tot_sectors, WORD(psect));
+			SET_INT(tot_sectors, WORD(psect));
 			This->serial_number = DWORD(serial);
 			nhs = DWORD(nhs);
 		} else {			
@@ -277,7 +277,7 @@ long getfree(Stream_t *Stream)
 {
 	DeclareThis(Fs_t);
 	register unsigned int i;
-	long total;
+	size_t total;
 
 	total = 0L;
 	for (i = 2; i < This->num_clus + 2; i++) {
