@@ -153,7 +153,7 @@ static struct directory *writeit(char *dosname,
 	long now;
 	int type, fat, ret;
 	long date;
-	unsigned long filesize, newsize;
+	size_t filesize, newsize;
 	Arg_t *arg = (Arg_t *) arg0;
 
 	if (arg->sc.File->Class->get_data(arg->sc.File,
