@@ -121,12 +121,12 @@ static inline void print_volume_label(Stream_t *Stream, char drive)
 
 static inline int list_directory(Stream_t *Dir, Stream_t *Fs, char *newname, 
 			  int wide, int all,
-			  long *tot_size)
+			  size_t *tot_size)
 {
 	int entry;
 	struct directory dir;
 	int files;
-	long size;
+	size_t size;
 	char shortname[13];
 	char longname[VBUFSIZE];
 	int i;
@@ -178,7 +178,7 @@ static inline int list_directory(Stream_t *Dir, Stream_t *Fs, char *newname,
 			if(dir.attr & 0x10)
 				printf("<DIR>    ");
 			else
-				printf(" %8ld", size);
+				printf(" %8ld", (long) size);
 			printf(" ");
 			print_date(&dir);
 			printf("  ");
@@ -197,7 +197,7 @@ static inline int list_directory(Stream_t *Dir, Stream_t *Fs, char *newname,
 }
 
 static inline void print_footer(char *drive, char *newname, int files,
-			 long tot_size, long blocks)
+			 size_t tot_size, size_t blocks)
 {	
 	char *s1,*s2;
 
@@ -225,7 +225,7 @@ void mdir(int argc, char **argv, int type)
 	char *arg;
 	StreamCache_t sc;
 	int i, files, fargn, wide, all, faked;
-	long blocks, tot_size=0;
+	size_t blocks, tot_size=0;
 	char last_drive;
 	char newpath[MAX_PATH];
 	char drive;
