@@ -110,15 +110,24 @@ static void init_geometry_boot(struct bootsector *boot, struct device *dev,
 
 static int comp_fat_bits(struct Fs_t *Fs, int fat_bits)
 {
-	if (fat_bits == 0 ){
-		if (Fs->num_clus >= FAT12)
-			Fs->fat_bits = 16;
-		else
-			Fs->fat_bits = 12;
-	} else if (fat_bits < 0) 
-		Fs->fat_bits = -fat_bits;
-	else
-		Fs->fat_bits = fat_bits;
+	switch(fat_bits) {
+		default:
+			fprintf(stderr,"Odd number of fat bits\n");
+		case 0:
+			if (Fs->num_clus >= FAT12)
+				set_fat16(Fs);
+			else
+				set_fat12(Fs);
+			break;
+		case 12:
+		case -12:
+			set_fat12(Fs);
+			break;
+		case 16:
+		case -16:
+			set_fat16(Fs);
+			break;			
+	}
 	return Fs->fat_bits;
 }
 
@@ -161,8 +170,7 @@ static inline void format_root(Fs_t *Fs, char *label, struct bootsector *boot)
 	}
 
 	memset(buf, '\0', Fs->sector_size);
-	WRITES(RootDir, buf, 0, Fs->sector_size);
-	for (i = 1; i < Fs->dir_len; i++)
+	for (i = 0; i < Fs->dir_len; i++)
 		WRITES(RootDir, buf, i * Fs->sector_size, Fs->sector_size);
 
 	ch.ignore_entry = 1;
@@ -185,6 +193,21 @@ static void calc_fat_size(Fs_t *Fs, int tot_sectors, int fat_bits)
 	 */
 	rem_sect = tot_sectors - Fs->dir_len - 1;
 	
+	if(Fs->fat_len) {
+		/* an XDF disk, we know the fat_size and have to find out the
+		 * rest. We start with a cluster size of 1 and keep doubling
+		 * until everything fits into the FAT. This will occur eventually,
+		 * as our FAT has a minimal size of 1 */
+		rem_sect -= 2 * Fs->fat_len;
+		for(Fs->cluster_size = 1; 1 ; Fs->cluster_size <<= 1) {
+			Fs->num_clus = rem_sect / Fs->cluster_size;
+			comp_fat_bits(Fs, fat_bits);
+			if (Fs->fat_len >= NEEDED_FAT_SIZE(Fs))
+				return;
+		}
+	}
+		
+
 	/* rough estimate of fat size */
 	Fs->fat_len = 1;	
 	tries=0;
@@ -197,7 +220,7 @@ static void calc_fat_size(Fs_t *Fs, int tot_sectors, int fat_bits)
 		
 		/* if we have exactly used up all
 		 * sectors, fine */
-		if ( occupied == rem_sect )
+		if ( rem_sect - occupied < Fs->cluster_size )
 			break;
 		
 		/* if we have used up more than we have,
@@ -223,7 +246,7 @@ static void calc_fat_size(Fs_t *Fs, int tot_sectors, int fat_bits)
 		case 2:
 			/* FALLTHROUGH */
 		case 1:
-			rem_sect--;
+			rem_sect-= Fs->cluster_size;
 		case 0:
 			continue;
 		}
@@ -264,7 +287,8 @@ struct OldDos_t old_dos[]={
 {   80,  9,  2, 7, 2, 3, 0xf9 },
 {   80, 15,  2,14, 1, 7, 0xf9 },
 {   80, 18,  2,14, 1, 9, 0xf0 },
-{   80, 36,  2,15, 2, 9, 0xf0 }
+{   80, 36,  2,15, 2, 9, 0xf0 },
+{    1,  8,  1, 1, 1, 1, 0xf0 },
 };
 
 static void calc_fs_parameters(struct device *dev,
@@ -296,18 +320,22 @@ static void calc_fs_parameters(struct device *dev,
 			boot->descr = 0xf8;
 		else
 			boot->descr = 0xf0;
-		if (dev->heads == 1)
-			Fs->cluster_size = 1;
-		else {
-			Fs->cluster_size = (tot_sectors > 2000 ) ? 1 : 2;
-			if (dev->use_2m & 0x7f)
+		if(!Fs->cluster_size) {
+			if (dev->heads == 1)
 				Fs->cluster_size = 1;
+			else {
+				Fs->cluster_size = (tot_sectors > 2000 ) ? 1:2;
+				if (dev->use_2m & 0x7f)
+					Fs->cluster_size = 1;
+			}
 		}
 
-		if (dev->heads == 1)
-			Fs->dir_len = 4;
-		else
-			Fs->dir_len = (tot_sectors > 2000) ? 10 : 7;
+		if(!Fs->dir_len) {
+			if (dev->heads == 1)
+				Fs->dir_len = 4;
+			else
+				Fs->dir_len = (tot_sectors > 2000) ? 11 : 7;
+		}			
 
 		Fs->num_clus = tot_sectors / Fs->cluster_size;
 		while (comp_fat_bits(Fs, dev->fat_bits) == 12 && 
@@ -346,7 +374,9 @@ void mformat(int argc, char **argv, int dummy)
 	int argssize=0; /* sector size */
 	int msize=0;
 #ifdef USE_XDF
+	int i;
 	int format_xdf = 0;
+	struct xdf_info info;
 #endif
 	struct bootsector *boot;
 	int c, oops;
@@ -358,7 +388,7 @@ void mformat(int argc, char **argv, int dummy)
 
 	char label[VBUFSIZE], buf[MAX_SECTOR], shortlabel[13];
 	struct device *dev;
-	char errmsg[80];
+	char errmsg[200];
 
 	unsigned long serial;
  	int serial_set;
@@ -376,17 +406,23 @@ void mformat(int argc, char **argv, int dummy)
 	serial_set = 0;
 	serial = 0;
 
+	Fs.cluster_size = 0;
 	Fs.refs = 1;
-	Fs.Class = &FsClass;
+	Fs.dir_len = 0;
+	Fs.fat_len = 0;
+	Fs.Class = &FsClass;	
 	rate_0 = mtools_rate_0;
 	rate_any = mtools_rate_any;
 
 	/* get command line options */
-	while ((c = getopt(argc, argv, "CXt:h:s:l:n:H:M:S:12:0Aa")) != EOF) {
+	while ((c = getopt(argc, argv, "Cc:Xt:h:s:l:n:H:M:S:12:0Aa")) != EOF) {
 		switch (c) {
 			case 'C':
 				create = O_CREAT;
 				break;
+			case 'c':
+				Fs.cluster_size = atoi(optarg);
+				break;
 			case 'H':
 				hs = atoi(optarg);
 				break;
@@ -476,11 +512,11 @@ void mformat(int argc, char **argv, int dummy)
 			continue;
 		used_dev = *dev;
 
-		set_uint(&used_dev.tracks, argtracks);
-		set_uint(&used_dev.heads, argheads);
-		set_uint(&used_dev.sectors, argsectors);
-		set_uint(&used_dev.use_2m, arguse_2m);
-		set_uint(&used_dev.ssize, argssize);
+		SET_INT(used_dev.tracks, argtracks);
+		SET_INT(used_dev.heads, argheads);
+		SET_INT(used_dev.sectors, argsectors);
+		SET_INT(used_dev.use_2m, arguse_2m);
+		SET_INT(used_dev.ssize, argssize);
 		
 		expand(dev->name, name);
 #ifdef USE_XDF
@@ -490,9 +526,15 @@ void mformat(int argc, char **argv, int dummy)
 						     O_RDWR | create,
 						     errmsg);
 #ifdef USE_XDF
-		else
+		else {
+			used_dev.use_xdf = 1;
 			Fs.Direct = Fs.Direct = XdfOpen(&used_dev, name, O_RDWR,
-							errmsg);
+							errmsg, &info);
+			if(Fs.Direct) {
+				Fs.dir_len = info.RootDirSize;
+				Fs.fat_len = info.FatSize;
+			}
+		}
 #endif
 
 		if (!Fs.Direct) {
@@ -521,7 +563,7 @@ void mformat(int argc, char **argv, int dummy)
 		Fs.sector_size = 512;
 		if( !(used_dev.use_2m & 0x7f))
 			Fs.sector_size = 128 << (used_dev.ssize & 0x7f);
-		set_uint(&Fs.sector_size, msize);
+		SET_INT(Fs.sector_size, msize);
 
 		/* do a "test" read */
 		if (!create &&
@@ -591,6 +633,14 @@ void mformat(int argc, char **argv, int dummy)
 	if(dev->use_2m & 0x7f)
 		Fs.num_fat = 1;
 	format_fat(&Fs, boot);
+#ifdef USE_XDF
+	if(format_xdf)
+		for(i=0; 
+		    i < (info.BadSectors+Fs.cluster_size-1)/Fs.cluster_size; 
+		    i++)
+			Fs.fat_encode(&Fs, i+2, 0xfff7);
+#endif
+
 	format_root(&Fs, label, boot);
 	WRITES((Stream_t *)&Fs, (char *) boot, 0, Fs.sector_size);	
 	FLUSH((Stream_t *)&Fs); /* flushes Fs. 
