@@ -42,6 +42,8 @@ struct directory {
 #define FILE_SIZE(dir)  (_DWORD((dir)->size))
 #define START(dir) (_WORD((dir)->start))
 
+/* ASSUMPTION: long is at least 32 bits */
+UNUSED(static inline void set_dword(unsigned char *data, unsigned long value));
 static inline void set_dword(unsigned char *data, unsigned long value)
 {
 	data[3] = (value >> 24) & 0xff;
@@ -50,6 +52,8 @@ static inline void set_dword(unsigned char *data, unsigned long value)
 	data[0] = (value >>  0) & 0xff;
 }
 
+/* ASSUMPTION: short is at least 16 bits */
+UNUSED(static inline void set_word(unsigned char *data, unsigned short value));
 static inline void set_word(unsigned char *data, unsigned short value)
 {
 	data[1] = (value >>  8) & 0xff;
