@@ -57,7 +57,11 @@ int lock_dev(int fd, int mode)
 #endif /* LOCKF */
 #endif /* FLOCK */
 	{
-		if(errno == EINVAL)
+		if(errno == EINVAL
+#ifdef  EOPNOTSUPP 
+		   || errno ==  EOPNOTSUPP
+#endif
+		  )
 			return 0;
 		else
 			return 1;
@@ -93,12 +97,12 @@ static int f_io(Stream_t *Stream, char *buf, int where, int len,
 	
 
 
-static int f_read(Stream_t *Stream, char *buf, int where, int len)
+static int f_read(Stream_t *Stream, char *buf, off_t where, size_t len)
 {	
 	return f_io(Stream, buf, where, len, (iofn) read);
 }
 
-static int f_write(Stream_t *Stream, char *buf, int where, int len)
+static int f_write(Stream_t *Stream, char *buf, off_t where, size_t len)
 {
 	return f_io(Stream, buf, where, len, (iofn) write);
 }
@@ -136,7 +140,7 @@ static int f_geom(Stream_t *Stream, struct device *dev,
 {
 	int ret;
 	DeclareThis(SimpleFloppy_t);
-	unsigned long tot_sectors;
+	size_t tot_sectors;
 	int BootP, Infp0, InfpX, InfTm;
 	int sectors, j;
 	unsigned char sum;
@@ -149,7 +153,7 @@ static int f_geom(Stream_t *Stream, struct device *dev,
 		dev->heads = CHAR(nheads);
 		dev->sectors = CHAR(nsect);
 		tot_sectors = DWORD(bigsect);
-		set_ulong(&tot_sectors, WORD(psect));
+		SET_INT(tot_sectors, WORD(psect));
 		sect_per_track = dev->heads * dev->sectors;
 		tot_sectors += sect_per_track - 1; /* round size up */
 		dev->tracks = tot_sectors / sect_per_track;
@@ -194,7 +198,7 @@ static int f_geom(Stream_t *Stream, struct device *dev,
 }
 
 
-static int f_data(Stream_t *Stream, long *date, unsigned long *size,
+static int f_data(Stream_t *Stream, long *date, size_t *size,
 		  int *type, int *address)
 {
 	DeclareThis(SimpleFloppy_t);
