@@ -162,7 +162,7 @@ int check_vfat(struct vfat_state *v, struct directory *dir)
 }
 
 
-void clear_vses(Stream_t *Dir, int entry, int last)
+void clear_vses(Stream_t *Dir, int entry, size_t last)
 {
 	struct directory dir;
 
